@@ -1606,7 +1606,11 @@ empathy_window_present_with_time (GtkWindow *window,
 			gtk_widget_hide (GTK_WIDGET (window));
 	}
 
-	gtk_window_present_with_time (window, timestamp);
+	if (timestamp == GDK_CURRENT_TIME)
+		gtk_window_present (window);
+	else
+		gtk_window_present_with_time (window, timestamp);
+
 	gtk_window_set_skip_taskbar_hint (window, FALSE);
 	gtk_window_deiconify (window);
 }
@@ -1614,7 +1618,7 @@ empathy_window_present_with_time (GtkWindow *window,
 void
 empathy_window_present (GtkWindow *window)
 {
-  empathy_window_present_with_time (window, GDK_CURRENT_TIME);
+  empathy_window_present_with_time (window, gtk_get_current_event_time ());
 }
 
 GtkWindow *
