@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include <telepathy-glib/account-channel-request.h>
 #include <telepathy-glib/util.h>
 #include <telepathy-glib/interfaces.h>
 
@@ -30,10 +31,12 @@
 
 #include "empathy-call-handler.h"
 #include "empathy-call-factory.h"
-#include "empathy-dispatcher.h"
 #include "empathy-marshal.h"
 #include "empathy-utils.h"
 
+#define DEBUG_FLAG EMPATHY_DEBUG_VOIP
+#include <libempathy/empathy-debug.h>
+
 G_DEFINE_TYPE(EmpathyCallHandler, empathy_call_handler, G_TYPE_OBJECT)
 
 /* signal enum */
@@ -44,6 +47,7 @@ enum {
   REQUEST_RESOURCE,
   CLOSED,
   STREAM_CLOSED,
+  CANDIDATES_CHANGED,
   LAST_SIGNAL
 };
 
@@ -59,6 +63,10 @@ enum {
   PROP_SEND_VIDEO_CODEC,
   PROP_RECV_AUDIO_CODECS,
   PROP_RECV_VIDEO_CODECS,
+  PROP_AUDIO_REMOTE_CANDIDATE,
+  PROP_VIDEO_REMOTE_CANDIDATE,
+  PROP_AUDIO_LOCAL_CANDIDATE,
+  PROP_VIDEO_LOCAL_CANDIDATE,
 };
 
 /* private structure */
@@ -75,6 +83,10 @@ typedef struct {
   FsCodec *send_video_codec;
   GList *recv_audio_codecs;
   GList *recv_video_codecs;
+  FsCandidate *audio_remote_candidate;
+  FsCandidate *video_remote_candidate;
+  FsCandidate *audio_local_candidate;
+  FsCandidate *video_local_candidate;
 } EmpathyCallHandlerPriv;
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyCallHandler)
@@ -121,6 +133,10 @@ empathy_call_handler_finalize (GObject *object)
   fs_codec_destroy (priv->send_video_codec);
   fs_codec_list_destroy (priv->recv_audio_codecs);
   fs_codec_list_destroy (priv->recv_video_codecs);
+  fs_candidate_destroy (priv->audio_remote_candidate);
+  fs_candidate_destroy (priv->video_remote_candidate);
+  fs_candidate_destroy (priv->audio_local_candidate);
+  fs_candidate_destroy (priv->video_local_candidate);
 
   if (G_OBJECT_CLASS (empathy_call_handler_parent_class)->finalize)
     G_OBJECT_CLASS (empathy_call_handler_parent_class)->finalize (object);
@@ -203,6 +219,18 @@ empathy_call_handler_get_property (GObject *object,
       case PROP_RECV_VIDEO_CODECS:
         g_value_set_boxed (value, priv->recv_video_codecs);
         break;
+      case PROP_AUDIO_REMOTE_CANDIDATE:
+        g_value_set_boxed (value, priv->audio_remote_candidate);
+        break;
+      case PROP_VIDEO_REMOTE_CANDIDATE:
+        g_value_set_boxed (value, priv->video_remote_candidate);
+        break;
+      case PROP_AUDIO_LOCAL_CANDIDATE:
+        g_value_set_boxed (value, priv->audio_local_candidate);
+        break;
+      case PROP_VIDEO_LOCAL_CANDIDATE:
+        g_value_set_boxed (value, priv->video_local_candidate);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
     }
@@ -277,6 +305,38 @@ empathy_call_handler_class_init (EmpathyCallHandlerClass *klass)
   g_object_class_install_property (object_class, PROP_RECV_VIDEO_CODECS,
     param_spec);
 
+  param_spec = g_param_spec_boxed ("audio-remote-candidate",
+    "audio remote candidate",
+    "Remote candidate used for the audio stream",
+    FS_TYPE_CANDIDATE,
+    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class,
+      PROP_AUDIO_REMOTE_CANDIDATE, param_spec);
+
+  param_spec = g_param_spec_boxed ("video-remote-candidate",
+    "video remote candidate",
+    "Remote candidate used for the video stream",
+    FS_TYPE_CANDIDATE,
+    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class,
+      PROP_VIDEO_REMOTE_CANDIDATE, param_spec);
+
+  param_spec = g_param_spec_boxed ("audio-local-candidate",
+    "audio local candidate",
+    "Local candidate used for the audio stream",
+    FS_TYPE_CANDIDATE,
+    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class,
+      PROP_AUDIO_REMOTE_CANDIDATE, param_spec);
+
+  param_spec = g_param_spec_boxed ("video-local-candidate",
+    "video local candidate",
+    "Local candidate used for the video stream",
+    FS_TYPE_CANDIDATE,
+    G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class,
+      PROP_VIDEO_REMOTE_CANDIDATE, param_spec);
+
   signals[CONFERENCE_ADDED] =
     g_signal_new ("conference-added", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
@@ -317,6 +377,12 @@ empathy_call_handler_class_init (EmpathyCallHandlerClass *klass)
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
       g_cclosure_marshal_VOID__OBJECT,
       G_TYPE_NONE, 1, TF_TYPE_STREAM);
+
+  signals[CANDIDATES_CHANGED] =
+    g_signal_new ("candidates-changed", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST, 0, NULL, NULL,
+      g_cclosure_marshal_VOID__UINT,
+      G_TYPE_NONE, 1, G_TYPE_UINT);
 }
 
 /**
@@ -400,6 +466,67 @@ update_receiving_codec (EmpathyCallHandler *self,
   g_object_unref (session);
 }
 
+static void
+update_candidates (EmpathyCallHandler *self,
+    FsCandidate *remote_candidate,
+    FsCandidate *local_candidate,
+    FsStream *stream)
+{
+  EmpathyCallHandlerPriv *priv = GET_PRIV (self);
+  FsSession *session;
+  FsMediaType type;
+
+  if (stream == NULL)
+    return;
+
+  g_object_get (stream, "session", &session, NULL);
+  if (session == NULL)
+    return;
+
+  g_object_get (session, "media-type", &type, NULL);
+
+  if (type == FS_MEDIA_TYPE_AUDIO)
+    {
+      if (remote_candidate != NULL)
+        {
+          fs_candidate_destroy (priv->audio_remote_candidate);
+          priv->audio_remote_candidate = fs_candidate_copy (remote_candidate);
+          g_object_notify (G_OBJECT (self), "audio-remote-candidate");
+        }
+
+      if (local_candidate != NULL)
+        {
+          fs_candidate_destroy (priv->audio_local_candidate);
+          priv->audio_local_candidate = fs_candidate_copy (local_candidate);
+          g_object_notify (G_OBJECT (self), "audio-local-candidate");
+        }
+
+      g_signal_emit (G_OBJECT (self), signals[CANDIDATES_CHANGED], 0,
+          FS_MEDIA_TYPE_AUDIO);
+    }
+  else if (type == FS_MEDIA_TYPE_VIDEO)
+    {
+      if (remote_candidate != NULL)
+        {
+          fs_candidate_destroy (priv->video_remote_candidate);
+          priv->video_remote_candidate = fs_candidate_copy (remote_candidate);
+          g_object_notify (G_OBJECT (self), "video-remote-candidate");
+        }
+
+      if (local_candidate != NULL)
+        {
+          fs_candidate_destroy (priv->video_local_candidate);
+          priv->video_local_candidate = fs_candidate_copy (local_candidate);
+          g_object_notify (G_OBJECT (self), "video-local-candidate");
+        }
+
+      g_signal_emit (G_OBJECT (self), signals[CANDIDATES_CHANGED], 0,
+          FS_MEDIA_TYPE_VIDEO);
+    }
+
+  g_object_unref (session);
+}
+
 void
 empathy_call_handler_bus_message (EmpathyCallHandler *handler,
   GstBus *bus, GstMessage *message)
@@ -440,6 +567,24 @@ empathy_call_handler_bus_message (EmpathyCallHandler *handler,
 
       update_receiving_codec (handler, codecs, stream);
     }
+  else if (s != NULL &&
+      gst_structure_has_name (s, "farsight-new-active-candidate-pair"))
+    {
+      const GValue *val;
+      FsCandidate *remote_candidate, *local_candidate;
+      FsStream *stream;
+
+      val = gst_structure_get_value (s, "remote-candidate");
+      remote_candidate = g_value_get_boxed (val);
+
+      val = gst_structure_get_value (s, "local-candidate");
+      local_candidate = g_value_get_boxed (val);
+
+      val = gst_structure_get_value (s, "stream");
+      stream = g_value_get_object (val);
+
+      update_candidates (handler, remote_candidate, local_candidate, stream);
+    }
 
   tf_channel_bus_message (priv->tfchannel, message);
 }
@@ -609,35 +754,45 @@ empathy_call_handler_start_tpfs (EmpathyCallHandler *self)
 }
 
 static void
-empathy_call_handler_request_cb (EmpathyDispatchOperation *operation,
-  const GError *error,
-  gpointer user_data)
+empathy_call_handler_request_cb (GObject *source,
+    GAsyncResult *result,
+    gpointer user_data)
 {
   EmpathyCallHandler *self = EMPATHY_CALL_HANDLER (user_data);
   EmpathyCallHandlerPriv *priv = GET_PRIV (self);
   TpChannel *channel;
+  GError *error = NULL;
+  TpAccountChannelRequest *req = TP_ACCOUNT_CHANNEL_REQUEST (source);
+  TpAccount *account;
 
-  if (error != NULL)
-    return;
+  channel = tp_account_channel_request_create_and_handle_channel_finish (req,
+      result, NULL, &error);
+  if (channel == NULL)
+    {
+      DEBUG ("Failed to create the channel: %s", error->message);
+      g_error_free (error);
+      return;
+    }
 
-  channel = empathy_dispatch_operation_get_channel (operation);
-  g_assert (channel != NULL);
+  account = tp_account_channel_request_get_account (req);
 
-  priv->call = empathy_tp_call_new (channel);
+  priv->call = empathy_tp_call_new (account, channel);
 
   g_object_notify (G_OBJECT (self), "tp-call");
 
   empathy_call_handler_start_tpfs (self);
 
-  empathy_dispatch_operation_claim (operation);
+  g_object_unref (channel);
 }
 
 void
 empathy_call_handler_start_call (EmpathyCallHandler *handler,
     gint64 timestamp)
 {
-
   EmpathyCallHandlerPriv *priv = GET_PRIV (handler);
+  TpAccountChannelRequest *req;
+  TpAccount *account;
+  GHashTable *request;
 
   if (priv->call != NULL)
     {
@@ -650,9 +805,17 @@ empathy_call_handler_start_call (EmpathyCallHandler *handler,
    * will be used to create a new EmpathyTpCall. */
   g_assert (priv->contact != NULL);
 
-  empathy_call_factory_new_call_with_streams (priv->contact,
-      priv->initial_audio, priv->initial_video, timestamp,
+  account = empathy_contact_get_account (priv->contact);
+  request = empathy_call_factory_create_request (priv->contact,
+      priv->initial_audio, priv->initial_video);
+
+  req = tp_account_channel_request_new (account, request, timestamp);
+
+  tp_account_channel_request_create_and_handle_channel_async (req, NULL,
       empathy_call_handler_request_cb, handler);
+
+  g_object_unref (req);
+  g_hash_table_unref (request);
 }
 
 /**
@@ -723,3 +886,39 @@ empathy_call_handler_get_recv_video_codecs (EmpathyCallHandler *self)
 
   return priv->recv_video_codecs;
 }
+
+FsCandidate *
+empathy_call_handler_get_audio_remote_candidate (
+    EmpathyCallHandler *self)
+{
+  EmpathyCallHandlerPriv *priv = GET_PRIV (self);
+
+  return priv->audio_remote_candidate;
+}
+
+FsCandidate *
+empathy_call_handler_get_audio_local_candidate (
+    EmpathyCallHandler *self)
+{
+  EmpathyCallHandlerPriv *priv = GET_PRIV (self);
+
+  return priv->audio_local_candidate;
+}
+
+FsCandidate *
+empathy_call_handler_get_video_remote_candidate (
+    EmpathyCallHandler *self)
+{
+  EmpathyCallHandlerPriv *priv = GET_PRIV (self);
+
+  return priv->video_remote_candidate;
+}
+
+FsCandidate *
+empathy_call_handler_get_video_local_candidate (
+    EmpathyCallHandler *self)
+{
+  EmpathyCallHandlerPriv *priv = GET_PRIV (self);
+
+  return priv->video_local_candidate;
+}
